module Com where

import Data.List

--最長重複文字列を計算する
maxDupStr :: [Char] -> [Char]
maxDupStr = extract . chooseMax . calcLen . makePair . sort . tails

--隣り合う要素を組にする
makePair :: [[Char]] -> [([Char], [Char])]
makePair xs = zip xs (tail xs)

--文字列の共通部分の長さを求める
calcLen :: [([Char], [Char])] -> [(Int, [Char])]
calcLen = map lenstr

lenstr :: ([Char],[Char]) -> (Int,[Char])
lenstr (xs,ys) = (comlen xs ys, xs)

comlen :: [Char] -> [Char] -> Int
comlen xs ys = length (takeWhile pairEq (zip xs ys))

pairEq :: (Char,Char) -> Bool
pairEq (x,y) = x == y

--共通部分が一番長い要素を取り出す
chooseMax :: [(Int,[Char])] -> (Int,[Char])
chooseMax = maximumBy compFst

compFst :: (Int,[Char]) -> (Int,[Char]) -> Ordering
compFst (n1,s1) (n2,s2) = compare n1 n2

--共通部分のみを残す
extract :: (Int, [Char]) -> [Char]
extract (i,xs) = take i xs

{-
> tails "mississippi"
["mississippi","ississippi","ssissippi","sissippi","issippi","ssippi","sippi","ippi","ppi","pi","i",""]

> (sort . tails) "mississippi"
["","i","ippi","issippi","ississippi","mississippi","pi","ppi","sippi","sissippi","ssippi","ssissippi"]

> (makePair . sort . tails) "mississippi"
[("","i"),("i","ippi"),("ippi","issippi"),("issippi","ississippi"),("ississippi","mississippi"),("mississippi","pi"),("pi","ppi"),("ppi","sippi"),("sippi","sissippi"),("sissippi","ssippi"),("ssippi","ssissippi")]

> (map lenstr . makePair . sort . tails) "mississippi"
[(0,""),(1,"i"),(1,"ippi"),(4,"issippi"),(0,"ississippi"),(0,"mississippi"),(1,"pi"),(0,"ppi"),(2,"sippi"),(1,"sissippi"),(3,"ssippi")]

> (chooseMax . map lenstr . makePair . sort . tails) "mississippi"
(4,"issippi")

> (extract . chooseMax . map lenstr . makePair . sort . tails) "mississippi"
"issi"

-}